-- return notes for sectorId
function PR.LoadSectorData(sectorId)
    return unspickle(LoadSystemNotes(PR.SystemNotesPrefix + sectorId)) or {}
end

-- does nothing if sector is already loaded
function PR.LoadSector(sectorId)
    if PR.LoadedSector == sectorId then return end -- only load sector once

    -- force pending sector data update if for a different sector
    PR.SaveNow(true)

    PR.LoadedSector = sectorId

    PR.Data = PR.LoadSectorData(PR.LoadedSector)
    if not PR.Data.MaxId then
        PR.Data.MaxId = 0 -- guaranteed to exist, 0 if no scanned asteroids
    end
    if not PR.Data.RoidList then
        PR.Data.RoidList = {} -- guaranteed to exist, empty if no scanned asteroids
    end

    -- get digits of max id for PR.FormatRoidId
    PR.MaxIdDigits = PR.GetDigitCount(PR.Data.MaxId)

    -- get count of unscanned asteroids
    PR.UpdateUnscannedCount()
end

-- saves the provided sector with optional data
function PR.SaveSectorData(sectorId, sectorData)
    sectorData = sectorData or {}
    if sectorId and sectorId > 0 then
        local unscanned = sectorData.Unscanned
        sectorData.Unscanned = nil -- no reason to store this in db
        SaveSystemNotes(spickle(sectorData), PR.SystemNotesPrefix + sectorId)
        sectorData.Unscanned = unscanned -- restore in memory value
    end
end

-- starts timer to save currently loaded sector
function PR.SaveSector()
    PR.SaveSectorId = PR.LoadedSector
    PR.SaveSectorTimer:SetTimeout(60000, PR.SaveSectorNow)
end

-- saves the currently loaded sector
function PR.SaveSectorNow()
    if PR.SaveSectorId == 0 then return end
    if PR.LeviDelayCount > 0 then
        PR.SaveSector()
        return -- dont save anything unless levi scanner done
    end
    if not PR.PluginReady or PR.Levi then return end
    if PR.SaveSectorTimer:IsActive() then PR.SaveSectorTimer:Kill() end
    if PR.SaveSectorId == PR.LoadedSector then
        PR.SaveSectorData(PR.LoadedSector, PR.Data)
    end
    PR.SaveSectorId = 0
end

-- permanently resets all data for current sector
function PR.ResetSector(sectorId)
    if sectorId and sectorId > 0 then
        -- disable the scan event while this runs to prevent race conditions
        UnregisterEvent(PR.TargetScanned, "TARGET_SCANNED")
        PR.SaveSectorId = 0
        if PR.SaveSectorTimer:IsActive() then PR.SaveSectorTimer:Kill() end
        PR.SaveSectorData(sectorId) -- write empty sector data
        PR.SystemUpdateSectorId = 0
        PR.LoadedSector = 0
        PR.LoadSector(sectorId)
        PR.SystemDataUpdate(sectorId)
        -- start scanning the newly virgin sector
        RegisterEvent(PR.TargetScanned, "TARGET_SCANNED")
        return true
    end
    return false
end

-- updates unscanned count
function PR.UpdateUnscannedCount()
    PR.Data.Unscanned = 0
    if PR.Data.MaxId > 0 then
        for i = 1, PR.Data.MaxId do
            if not PR.IsScanned(i) then
                PR.Data.Unscanned = PR.Data.Unscanned + 1
            end
        end
    end
end

-- returns table of ores in sector {{ ore.shortname = ore.name }} or nil
function PR.GetSectorOres()
    local sector_ores = {}
    local system_data = PR.LoadSystemData(GetSystemID(PR.LoadedSector))
    if next(system_data) then
        for i = 1, PR.Ore.n do
            if system_data[PR.Ore[i].shortname][PR.LoadedSector] then
                table.insert(sector_ores,
                    { shortname = PR.Ore[i].shortname, name = PR.Ore[i].name })
            end
        end
    end
    sector_ores.n = #sector_ores
    if sector_ores.n > 0 then return sector_ores end
end
